package com.ruoyi.system.mapper;

import com.ruoyi.system.entity.CommonEntity;
import com.ruoyi.system.entity.DragTableCondition;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 动态表格条件Mapper接口
 * 
 * @author ruoyi
 * @date 2023-07-31
 */
public interface DragTableConditionMapper 
{
    /**
     * 查询动态表格条件列表
     */
    List<CommonEntity> selectDragTableConditionList(@Param("conditions") Map<String, Object> conditions);

    /**
     * 新增动态表格条件
     */
    int insertDragTableCondition(@Param("fieldNames") Set<String> fieldNames,@Param("fieldValues") List<Map<String, Object>> fieldValues);

    /**
     * 新增动态表格条件
     */
    int insertDragTableConditionByList(List<DragTableCondition> conditionList);

    /**
     * 修改动态表格条件
     */
    int updateDragTableCondition(@Param("fields") Map<String, Object> fields, @Param("conditions") Map<String, Object> conditions);

    /**
     * 批量删除动态表格条件
     * 
     * @param tcIds 需要删除的数据主键集合
     * @return 结果
     */
    int deleteDragTableConditionByTcIds(@Param("conditions") Map<String, Object> conditions);

    /**
     * 查询动态表格条件
     *
     * @param tId 动态表格条件主键
     * @return 动态表格条件
     */
    List<DragTableCondition> selectDragTableConditionByTid(Long tId);

    /**
     * 查询动态表格超级查询范围字段
     */
    List<String> selectDragTableSearchField(Long tId);

    /**
     * 删除动态表格条件
     * @param tIds 动态表格编号
     * @return
     */
    int deleteDragTableConditionBytIds(List<Long> tIds);

}
